"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.preprocessReadme = void 0;
var compiler_1 = require("svelte/compiler");
var markdown_it_1 = __importDefault(require("markdown-it"));
var markdown_it_anchor_1 = __importDefault(require("markdown-it-anchor"));
var prettier_1 = __importDefault(require("prettier"));
var prismjs_1 = __importDefault(require("prismjs"));
require("prismjs/components/prism-bash");
require("prismjs/components/prism-typescript");
require("prismjs/components/prism-jsx");
require("prismjs/components/prism-yaml");
require("prism-svelte");
var is_relative_url_1 = __importDefault(require("is-relative-url"));
var url_1 = require("url");
var aliases = {
    sh: "bash",
    js: "javascript",
    ts: "typescript",
    tsx: "typescript",
    yml: "yaml"
};
var md;
var getChildNodeText = function (node) {
    return node.children
        .flatMap(function (child) { return (child.type === "Element" ? child.children : child); })
        .filter(function (child) { return child.type === "Text"; })
        .map(function (child) { return child.raw; })
        .join("");
};
function preprocessReadme(opts) {
    var prefixUrl = opts.prefixUrl || "".concat(opts.homepage, "/tree/master/");
    var script_content = [];
    if (!md) {
        md = new markdown_it_1["default"]({
            html: true,
            linkify: true,
            typographer: true,
            highlight: function (source, lang, attrs) {
                if (lang === "svelte") {
                    var noEval = /no-eval/.test(attrs);
                    var noDisplay = /no-display/.test(attrs);
                    var instance = (0, compiler_1.parse)(source).instance;
                    if (instance !== undefined && !noEval) {
                        script_content = __spreadArray(__spreadArray([], __read(script_content), false), __read(source
                            .slice(instance.start, instance.end)
                            .split("\n")
                            .slice(1, -1)
                            .map(function (line) { return line.trim().replace(new RegExp(opts.name, "g"), opts.svelte); })), false);
                    }
                    var regex = new RegExp('"' + opts.name + '"', "g");
                    var modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
                    var formattedCode = prettier_1["default"].format(source, {
                        parser: "svelte"
                    });
                    var svelteCode = prismjs_1["default"].highlight(formattedCode, prismjs_1["default"].languages.svelte, "svelte");
                    return "<pre class=\"language-".concat(lang, "\" ").concat(noEval || noDisplay ? "" : "data-svelte=\"".concat(modifiedSource, "\""), ">{@html `").concat(svelteCode, "`}</pre>");
                }
                try {
                    var alias_lang = aliases[lang] || lang;
                    return "<pre class=\"language-".concat(alias_lang, "\">{@html `").concat(prismjs_1["default"].highlight(source, prismjs_1["default"].languages[alias_lang], alias_lang), "`}</pre>");
                }
                catch (e) {
                    console.error("Could not highlight language \"".concat(lang, "\"."));
                    return "<pre class=\"language-".concat(lang, "\">{@html `").concat(source, "`}</pre>");
                }
            }
        });
        md.use(markdown_it_anchor_1["default"]);
    }
    return {
        // @ts-ignore
        markup: function (_a) {
            var content = _a.content, filename = _a.filename;
            if (filename && (/node_modules/.test(filename) || !filename.endsWith(".md")))
                return null;
            if (opts.repoUrl) {
                content = content.replace("<!-- REPO_URL -->", "[GitHub repo](".concat(opts.repoUrl, ")"));
            }
            content = content.replace("<!-- TOC -->", "\n## Table of Contents\n      ");
            var style_content = "";
            var result = md.render(content);
            var cursor = 0;
            var ast = (0, compiler_1.parse)(result);
            var headings = [];
            var prev = undefined;
            (0, compiler_1.walk)(ast, {
                enter: function (node, parent) {
                    if (node.type === "Attribute" && node.name === "href") {
                        var value = node.value[0];
                        if (value && !value.raw.startsWith("#") && (0, is_relative_url_1["default"])(value.raw)) {
                            var relative_path = new url_1.URL(value.raw, prefixUrl).href;
                            result = result.replace(value.raw, relative_path);
                            cursor += relative_path.length - value.raw.length;
                        }
                    }
                    if (node.type === "Style") {
                        style_content += result.slice(node.content.start, node.content.end);
                        var replace_style = result.slice(node.start + cursor, node.end + cursor);
                        result = result.replace(replace_style, "");
                        cursor -= replace_style.length;
                    }
                    if (node.type === "Element" && node.name === "h2") {
                        // @ts-ignore
                        var id = node.attributes.find(function (attr) { return attr.name === "id"; }).value[0].raw;
                        if (id === "table-of-contents")
                            return;
                        var text = getChildNodeText(node);
                        if (text !== undefined) {
                            if (prev === "h3") {
                                headings.push("</ul><li><a href=\"#".concat(id, "\">").concat(text, "</a></li>"));
                            }
                            else {
                                headings.push("<li><a href=\"#".concat(id, "\">").concat(text, "</a></li>"));
                            }
                            prev = "h2";
                        }
                    }
                    if (node.type === "Element" && node.name === "h3") {
                        // @ts-ignore
                        var id = node.attributes.find(function (attr) { return attr.name === "id"; }).value[0].raw;
                        var text = getChildNodeText(node);
                        if (text !== undefined) {
                            if (prev === "h2") {
                                headings.push("<ul><li><a href=\"#".concat(id, "\">").concat(text, "</a></li>"));
                            }
                            else {
                                headings.push("<li><a href=\"#".concat(id, "\">").concat(text, "</a></li>"));
                            }
                            prev = "h3";
                        }
                    }
                    if (node.type === "Attribute" && node.name === "data-svelte") {
                        var raw_value = node.value[0].raw;
                        var value = decodeURI(raw_value);
                        var value_ast = (0, compiler_1.parse)(value);
                        var markup = "<div class=\"code-fence\">" + value.slice(value_ast.html.start, value_ast.html.end) + "</div>";
                        var replace = result.slice(parent.start + cursor, parent.end + cursor);
                        result = result.replace(replace, markup + replace.replace(raw_value, ""));
                        cursor += markup.length - raw_value.length;
                    }
                }
            });
            if (prev === "h3") {
                headings.push("</ul>");
            }
            result = result.replace("<h2 id=\"table-of-contents\">Table of Contents</h2>", "<p><strong>Table of Contents</strong></p><ul>".concat(headings.join("\n"), "</ul>"));
            return {
                code: "<script>".concat(__spreadArray([], __read(new Set(script_content)), false).join(""), "</script>\n               <style>").concat(style_content, "</style>\n               <main class=\"markdown-body\">").concat(result, "</main>")
            };
        }
    };
}
exports.preprocessReadme = preprocessReadme;
